


## Import dependencies
import requests
import pandas as pd
import numpy as np
import time
from datetime import datetime
from bs4  import BeautifulSoup
import re
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
#from tqdm import tqdm
#from tableauhyperapi import (
#    HyperProcess,
#    Telemetry,
#    Connection,
#    SqlType,
#    TableDefinition,
#    CreateMode,
#    TableName,
#    Inserter,
#)



## Starting requests session with proper settings
def s_session():
    
    s = requests.Session()
    
    retry = Retry(connect = 3, backoff_factor = 0.5)
    
    adapter = HTTPAdapter(max_retries = retry)
    
    s.mount('http://', adapter)
    
    s.mount('https://', adapter)
    
    return s

## Function to convert game ids to html versions
def convert_ids(api_game_id):
    
    html_season_id = str(int(str(api_game_id)[:4])) + str(int(str(api_game_id)[:4]) + 1)
    
    html_game_id = str(api_game_id)[5:]
    
    return html_season_id, html_game_id

## Printing game ID times
def print_game_id_time(game_start, game_id):
    
    timer = round(time.perf_counter() - game_start, 2)
    
    return print(f'Finished scraping {game_id} in {timer} seconds...')

## Print scrape intro
def print_scrape_intro(game_ids, scrape_type):
    
    number_of_games = len(game_ids)
    
    current_time = datetime.now().strftime("%H:%M")

    if number_of_games > 1:

        number = 'games'

    else:

        number = 'game'

    return print(f'Starting {scrape_type} scrape for {number_of_games} {number} at {current_time}\n')

## Printing time to concat
def print_concat_finish(concat_start):
    
    timer = round(time.perf_counter() - concat_start, 2)

    message = 'Finished concatenating in '

    if timer > 60:

        return print(message + f'{round(timer / 60, 2)} minutes...')

    else:

        return print(message + f'{timer} seconds...')

## Converting time to minutes or seconds at the end of a function
def end_timer(time_difference, message):
    
    """
    More TBD docstring
    """

    if time_difference > 60:
        
        return print(message.strip() + f' {round(time_difference / 60, 2)} minutes\n')
    
    else:
        
        return print(message.strip() + f' {round(time_difference, 2)} seconds\n')

## Printing time to console
def print_number_of_games(game_ids, scrape_start):

    number_of_games = len(game_ids)

    timer = round(time.perf_counter() - scrape_start, 2)

    current_time = datetime.now().strftime("%H:%M")
    
    if number_of_games > 1:

        number = 'games'

    else:

        number = 'game'

    message = f'\nFinished scraping {number_of_games} {number} in '

    if timer > 60:

        print(message + f'{round(timer / 60, 2)} minutes at {current_time}\n')

    else:

        print(message + f'{timer} seconds at {current_time}\n')

## If not a list, convert to list
def convert_to_list(obj):
    
    """
    More TBD docstring
    """
    
    ## Convert game IDs to strings if integers. If given a single game ID, convert to list of game IDs. 
    if type(obj) is not list and type(obj) is not pd.Series:
        
        obj = [obj]
    
    return obj

## Something from TDH
def hs_strip_html(td):
    """
    Function from Harry Shomer's Github
    
    Strip html tags and such 
    
    :param td: pbp
    
    :return: list of plays (which contain a list of info) stripped of html
    """
    for y in range(len(td)):
        # Get the 'br' tag for the time column...this get's us time remaining instead of elapsed and remaining combined
        if y == 3:
            td[y] = td[y].get_text()   # This gets us elapsed and remaining combined-< 3:0017:00
            index = td[y].find(':')
            td[y] = td[y][:index+3]
        elif (y == 6 or y == 7) and td[0] != '#':
            # 6 & 7-> These are the player 1 ice one's
            # The second statement controls for when it's just a header
            baz = td[y].find_all('td')
            bar = [baz[z] for z in range(len(baz)) if z % 4 != 0]  # Because of previous step we get repeats...delete some

            # The setup in the list is now: Name/Number->Position->Blank...and repeat
            # Now strip all the html
            players = []
            for i in range(len(bar)):
                if i % 3 == 0:
                    try:
                        name = return_name_html(bar[i].find('font')['title'])
                        number = bar[i].get_text().strip('\n')  # Get number and strip leading/trailing newlines
                    except KeyError:
                        name = ''
                        number = ''
                elif i % 3 == 1:
                    if name != '':
                        position = bar[i].get_text()
                        players.append([name, number, position])

            td[y] = players
        else:
            td[y] = td[y].get_text()

    return td

## below are tableau functions
# def fix_dtypes(df):

#     dtype_mapper = {
#     "string": SqlType.text(),
#     "str": SqlType.text(),
#     "object": SqlType.text(),
#     "O": SqlType.text(),
#     "int64": SqlType.big_int(),
#     "int32": SqlType.big_int(),
#     "int16": SqlType.big_int(),
#     "int8": SqlType.big_int(),
#     "float32": SqlType.double(),
#     "float64": SqlType.double(),
#     "bool": SqlType.bool(),
#     "boolean": SqlType.bool(),
#     "datetime64[ns]": SqlType.timestamp(),
#     "timedelta[ns]": SqlType.interval(),
#     "category": SqlType.text(),
#     }


#     df = df.replace(to_replace = '', value = None)

#     for col in df.columns:
        
#         if df[col].dtype == 'O':
#             df = df.astype({col: 'str'}, errors = 'ignore')
#             pd.to_numeric(col, errors = 'ignore')
#     return df


# def to_hyper(dfs, hyper_file_name, custom_table_names=["Extract"], custom_schema="Extract"):
#     """
#     Write a Tableau Hyper file from a Pandas DataFrame.
#     Args:
#         df: Specify which DataFrame or list of DataFrames you want to output
#         hyper_file_name: Specify the file name such as "Example.hyper"
#         custom_schema: If you need to change the schema name. Defaults to "Extract"
#         custom_table_name: Name or names of the given DataFrames
#     Returns:
#         Tableau Hyper file
#     """

#     # Check if dfs and xustom_table_names have the right format
#     if type(dfs) is not list:
#         dfs = [dfs]
#     if type(custom_table_names) is not list:
#         custom_table_names = [custom_table_names]    
        
#     # Starts the Hyper Process
#     with HyperProcess(
#         telemetry=Telemetry.DO_NOT_SEND_USAGE_DATA_TO_TABLEAU,
#         parameters={"log_config": ""},
#     ) as hyper:

#         # Creates a .hyper file to put the data into
#         with Connection(
#             hyper.endpoint, hyper_file_name, CreateMode.CREATE_AND_REPLACE
#         ) as connection:

#             connection.catalog.create_schema(custom_schema)

#             # Creates one table for each DataFrame
#             for df, name in zip(dfs, custom_table_names):

#                 # create a .hyper compatible column definition
#                 # from pd DataFrame column names and dtypes
#                 # using 3 list comprehensions to loop through
#                 # all the columns in the DataFrame

#                 column_names = [column for column in df.columns]

#                 column_dtype = [dtype for dtype in df.dtypes]

#                 hyper_table = TableDefinition(
#                     TableName(custom_schema, name),
#                     [
#                         TableDefinition.Column(
#                             column_names[column], dtype_mapper[str(column_dtype[column]).lower()]
#                         )
#                         for column in range(len(column_names))
#                     ],
#                 )
#                 connection.catalog.create_table(hyper_table)

#                 # Repace NaN with None, otherwise it will not be Null in Tableau
#                 df.replace({np.nan: None}, inplace=True)

#                 # Insert the data values into the hyper file
#                 data_to_insert = df.to_numpy()
#                 with Inserter(connection, hyper_table) as inserter:
#                     inserter.add_rows(tqdm((row for row in data_to_insert)))
#                     inserter.execute()